/*
 * Project name:
     Frequency_on_LCD
 * Copyright:
     (c) MikroElektronika, 2009.
 * Revision History:
     20090715:
       - initial release by Mileta Miletic
 * Description:
     This project demonstrates working with TSL230BR Light-to-frequency converter.
     Pulses from TSL230BR are counted using RB0/INT interrupt on rising edge
     on pin RB0 of 16F876A MCU. Timer1 generates interrupt after 1/4 second
     for calculating frequency and displaying it on LCD.
 * Test configuration:
     MCU:             PIC16F876A
                      http://ww1.microchip.com/downloads/en/DeviceDoc/41291F.pdf
     Dev.Board:       EasyPIC5
                      http://www.mikroe.com/en/tools/easypic5/
     Oscillator:      HS, 08.000 MHz
     Ext. Modules:    Light To Frequency 2 board on PORTB, LCD 2x16 on PORTD
                      http://www.mikroe.com/en/tools/lighttofreq2/
     SW:              mikroC PRO for PIC v3.80
                      http://www.mikroe.com/en/compilers/mikroc/pro/pic/
 * NOTES:

     RB0 ->     FREQ-OUT  (1)
     RB1 -> S0            (2)
     RB2 ->          /CS  (4)
     RB3 -> S1            (8)
     RB4                  (16)
     RB5 -> S2            (32)
     RB6                  (64)
     RB7 -> S3            (128)
     
     TSL230BR     MIN   TYP   MAX
                  95    100   105 kHz
     Operating Characteristics at VDD = 5 V, TA = 25C, Ee = 130 W/cm2
     lp = 640 nm (unless otherwise noted)
     
     Re = 0.77KHz / (J/cm)
     ==> J/cm = frequency(KHz) / 0.77 = frequency(KHz) * 1.2987012987012987012987012987013
     
     Ee = (frequency/1000) * 1.2987012987012987 * (output scaling) * (sensitivity/100)
     Ee = frequency * (output scaling) * (sensitivity) * 1.2987012987012987 / 100000
     Ee = frequency * (output scaling) * (sensitivity) * 1.2987012987012987e-5
     where      Ee = irradiance in W/cm
 */
// CS_PIN
#define CS_TSL230_ENABLE        0
#define CS_TSL230_DISABLE       4
// SENSITIVITY avec S1 & S0
#define SENSITIVITY_Power_Down  0
#define SENSITIVITY_1x          2
#define SENSITIVITY_10x         8
#define SENSITIVITY_100x        10

#define k_SENSITIVITY_1x          1.3e2
#define k_SENSITIVITY_10x         1.3e1
#define k_SENSITIVITY_100x        1.3e0
// Fo Scaling avec S3 & S2
#define Fo_Scaling_1x           0
#define Fo_Scaling_2x           32
#define Fo_Scaling_10x          128
#define Fo_Scaling_100x         160

#define k_Fo_Scaling_1x           1.
#define k_Fo_Scaling_2x           0.5
#define k_Fo_Scaling_10x          0.1
#define k_Fo_Scaling_100x         0.01
 /*
// LCD module connections
sbit LCD_D4 at RD0_bit;
sbit LCD_D5 at RD2_bit;
sbit LCD_D6 at RD4_bit;
sbit LCD_D7 at RD6_bit;

sbit LCD_RS at RD7_bit;
sbit LCD_EN at RD5_bit;

sbit LCD_D4_Direction at TRISD0_bit;
sbit LCD_D5_Direction at TRISD2_bit;
sbit LCD_D6_Direction at TRISD4_bit;
sbit LCD_D7_Direction at TRISD6_bit;

sbit LCD_RS_Direction at TRISD7_bit;
sbit LCD_EN_Direction at TRISD5_bit;
        */
unsigned long   cntr = 0;  // number of RB0 transition unsigned
int    ovrflw ;            // number of timer0 overflows
//unsigned long   seconde = 0;  // number of  seconde
unsigned long freq;
char txt[16];

// interrupt routine called 2000000 / 256 times by seconds :
// the timer TMR0 is increased each 4 clock cycles (quartz frequency is 8 Mhz),
// and overflows when reseting from 255 to 0,
// calling the interrupt procedure with bit T0IF set
// also called on each RBO transition, with bit INTF set
void interrupt(void)
{        
 if(INTCON.INTF)
 {
  // RB0 interrupt
  cntr++ ;                // inc. transition counter
  INTCON.INTF = 0 ;       // clear interrupt flag to enable next call
 }
 else
   if(INTCON.T0IF)
 {
  // TIMER 0 overflow
  ovrflw++ ;
  // inc. overflow counter
  INTCON.T0IF = 0 ;  // clear interrupt flag to enable next call on overflow
 }
}

//unsigned long ee;
float ee;
char uart_rd;

void main()
{
  freq      = 0;
  //seconde = 0;

  PORTB     = 0;          // Clear PORTB
  TRISB     = 0x01 ;      // RB0 pin as input, pins as outputs
    /*
  PORTD     = 0;          // Clear PORTD
  TRISD     = 0x00;       // Set PORTD pins as outputs
    */
    /*
  ANSELH    = 0;          // Configure ANS8..ANS13 pins as digital I/O
  C1ON_bit  = 0;          // Disable comparators
  C2ON_bit  = 0;
  */
    /*
  Lcd_Init();             // Initialize Lcd module
  Lcd_Cmd(_LCD_CLEAR);                  // Clear display
  Lcd_Cmd(_LCD_CURSOR_OFF);             // Turn cursor off
    */
  OPTION_REG = 0b11011000 ;       // no prescaler
  
  UART1_Init(9600);               // Initialize UART module at 9600 bps
  Delay_ms(100);                  // Wait for UART module to stabilize

  UART1_Write_Text("Start");
  UART1_Write(10);
  UART1_Write(13);

  // TSL230BR settings, check TSL230BR datasheet
  // S0 = S1 = S2 = S3 = 1 and CS = 0
  // SENSITIVITY : 100x
  // Fo Scaling : 100x
  // TSL230BR  Actif
  //
  PORTB      = SENSITIVITY_100x | Fo_Scaling_100x | CS_TSL230_ENABLE;
  
  while(1)     // Endless loop
  {
   cntr = 0 ;              // clear counters
   ovrflw = 0 ;

   INTCON = 0b10110110 ;           // GIE, TOIE, INTE, T0IF and INTF enabled

   while(ovrflw < 7813) ;         // wait 1 second : 7813 = 8 000 000 / 4 / 256, rounded up

      INTCON.GIE = 0 ;                // stop all interrupts
      //seconde++;
      freq = cntr;
      //ee = 1.2987012987012987e-5 * (float)freq;
      //      freq(Hz)       1.3e3 W/cm
      ee = (float)freq * 0.001 * k_SENSITIVITY_100x * k_Fo_Scaling_100x;
      
      /*
      //LongToStr(seconde, txt);    // Convert Time (long integer number) to a string
      //Lcd_Out(1, 1, txt);         // Print Time on Lcd
      //Lcd_Out_Cp(" s");           // Print text on Lcd

      floatToStr(ee, txt);    // Convert irradiance (float number) to a string
      Lcd_Out(1, 1, txt);         // Print irradiance on Lcd
      Lcd_Out_Cp(" uW/cm2");           // Print text on Lcd
      // Ee = frequency * (output scaling) * (sensitivity) * 7.7e-6
      // Pour SENSITIVITY = 100x et Fo Scaling = 100x  ont  :
      // Ee(W/cm) = frequency(Hz) * 1.2987012987012987e-5

      LongToStr(freq, txt);       // Convert freq (long integer number) to a string
      Lcd_Out(2, 1, txt);         // Print frequency on Lcd
      Lcd_Out_Cp(" Hz");//(2, 10,"Hz");        // Print text on Lcd
       */
      
     /*if (UART1_Data_Ready())  // If data is received,
     { */
      //uart_rd = UART1_Read();     // read the received data,
      //UART1_Write(uart_rd);       // and send data via UART
      FloatToStr(ee, txt);    // Convert irradiance (float number) to a string
      UART1_Write_Text(txt);         // Print irradiance on Lcd
      UART1_Write_Text(" uW/cm2 [ ");           // Print text on Lcd
      
      LongToStr(freq, txt);       // Convert freq (long integer number) to a string
      UART1_Write_Text(txt);         // Print frequency on Lcd
      UART1_Write_Text(" Hz ] \n\r");//(2, 10,"Hz");        // Print text on Lcd
      
      //UART1_Write(10);
      //UART1_Write(13);
     //}
  }
}